import { ethers, Signature } from 'ethers';
import { SignedState } from '../signatures';
import { FixedPart, State, VariablePart } from './state';
import { Address, Bytes32, Uint8, Uint48 } from './types';
export declare function hashChallengeMessage(challengeState: State): Bytes32;
/**
 * Holds information from a ChallengeRegistered event in a convenient form
 */
export interface ChallengeRegisteredEvent {
    channelId: Bytes32;
    finalizesAt: number;
    challengeStates: SignedState[];
}
export interface ChallengeRegisteredStruct {
    channelId: Bytes32;
    turnNumRecord: Uint48;
    finalizesAt: Uint48;
    challenger: Address;
    isFinal: boolean;
    fixedPart: Array<any>;
    variableParts: Array<any>;
    sigs: Signature[];
    whoSignedWhat: Uint8[];
}
/**
 * Extracts a ChallengeRegisteredEvent (containing challengeStates) from the supplied eventResult.
 * @param eventResult the event itself
 * @returns a ChallengeRegisteredEvent
 */
export declare function getChallengeRegisteredEvent(eventResult: any[]): ChallengeRegisteredEvent;
export interface ChallengeClearedEvent {
    kind: 'respond' | 'checkpoint';
    newStates: SignedState[];
}
export interface ChallengeClearedStruct {
    channelId: string;
    newTurnNumRecord: string;
}
export interface RespondTransactionArguments {
    challenger: string;
    isFinalAb: [boolean, boolean];
    fixedPart: FixedPart;
    variablePartAB: [VariablePart, VariablePart];
    sig: Signature;
}
/**
 * Extracts a ChallengeClearedEvent (containing a new signedState) from the logs of a respond or checkpoint transaction
 * @param tx a suitable transaction causing a ChallengeCleared event to be emitted
 * @param eventResult the event itself
 * @returns a ChallengeClearedEvent
 */
export declare function getChallengeClearedEvent(tx: ethers.Transaction, eventResult: any[]): ChallengeClearedEvent;
