"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getChannelId = exports.isExternalDestination = void 0;
const ethers_1 = require("ethers");
/**
 * Determines if the supplied 32 byte hex string represents an external destination (meaning funds will be paid _out_ of the adjudicator on chain)
 * @param bytes32 a destination
 * @returns true if the destination has 12 leading bytes as zero, false otherwise
 */
function isExternalDestination(bytes32) {
    return /^0x(0{24})([a-fA-F0-9]{40})$/.test(bytes32);
}
exports.isExternalDestination = isExternalDestination;
/**
 * Computes the unique id for the supplied channel
 * @param channel Parameters which determine the id
 * @returns a 32 byte hex string representing the id
 */
function getChannelId(fixedPart) {
    const { participants, channelNonce, appDefinition, challengeDuration } = fixedPart;
    const channelId = ethers_1.utils.keccak256(ethers_1.utils.defaultAbiCoder.encode(['address[]', 'uint256', 'address', 'uint48'], [participants, channelNonce, appDefinition, challengeDuration]));
    if (isExternalDestination(channelId))
        throw Error('This channel would have an external destination as an id');
    return channelId;
}
exports.getChannelId = getChannelId;
