"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createValidTransitionTransaction = exports.validTransition = exports.ForceMoveAppContractInterface = void 0;
const ethers_1 = require("ethers");
const IForceMoveApp_json_1 = __importDefault(require("../../artifacts/contracts/interfaces/IForceMoveApp.sol/IForceMoveApp.json"));
const state_1 = require("../contract/state");
//  https://github.com/ethers-io/ethers.js/issues/602#issuecomment-574671078
exports.ForceMoveAppContractInterface = new ethers_1.utils.Interface(IForceMoveApp_json_1.default.abi);
/**
 * Calls the valiTransition method on the supplied ForceMoveApp using eth_call
 * @param fromState a State
 * @param toState a State
 * @param appContract a ForceMoveApp contract address
 * @returns a Promise that resolves to a boolean
 */
async function validTransition(fromState, toState, appContract) {
    const numberOfParticipants = toState.participants.length;
    const fromVariablePart = (0, state_1.getVariablePart)(fromState);
    const toVariablePart = (0, state_1.getVariablePart)(toState);
    return await appContract.validTransition(fromVariablePart, toVariablePart, numberOfParticipants);
}
exports.validTransition = validTransition;
/**
 * Encodes a validTransition method call as the data on an ethereum transaction. Useful for testing gas consumption of a ForceMoveApp.
 */
function createValidTransitionTransaction(fromState, toState) {
    const numberOfParticipants = toState.participants.length;
    const fromVariablePart = (0, state_1.getVariablePart)(fromState);
    const toVariablePart = (0, state_1.getVariablePart)(toState);
    const data = exports.ForceMoveAppContractInterface.encodeFunctionData('validTransition', [
        fromVariablePart,
        toVariablePart,
        numberOfParticipants,
    ]);
    return { data };
}
exports.createValidTransitionTransaction = createValidTransitionTransaction;
