import { BigNumber } from 'ethers';
import ExitFormat from '@statechannels/exit-format';
/**
 * Holds rich information about a Deposited event emitted on chain
 */
export interface DepositedEvent {
    destination: string;
    destinationHoldings: BigNumber;
}
/**
 * Extracts a DepositedEvent from a suitable eventResult
 * @param eventResult
 * @returns a DepositedEvent
 */
export declare function getDepositedEvent(eventResult: any[]): DepositedEvent;
/**
 * Extracts a 20 byte address hex string from a 32 byte hex string by slicing
 * @param bytes32 a hex string
 * @returns a copy of the hex string with the first 12 bytes removed (after the "0x")
 */
export declare function convertBytes32ToAddress(bytes32: string): string;
/**
 * Left pads a 20 byte address hex string with zeros until it is a 32 byte hex string
 * e.g.,
 * 0x9546E319878D2ca7a21b481F873681DF344E0Df8 becomes
 * 0x0000000000000000000000009546E319878D2ca7a21b481F873681DF344E0Df8
 * @param address 20 byte hex string
 * @returns 32 byte padded hex string
 */
export declare function convertAddressToBytes32(address: string): string;
/**
 *
 * Emulates solidity code. TODO replace with PureEVM implementation?
 * @param initialHoldings
 * @param allocation
 * @param indices
 */
export declare function computeReclaimEffects(sourceAllocations: ExitFormat.Allocation[], // we must index this with a JS number that is less than 2**32 - 1
targetAllocations: ExitFormat.Allocation[], // we must index this with a JS number that is less than 2**32 - 1
indexOfTargetInSource: number): ExitFormat.Allocation[];
/**
 *
 * Emulates solidity code. TODO replace with PureEVM implementation?
 * @param initialHoldings
 * @param allocation
 * @param indices
 */
export declare function computeTransferEffectsAndInteractions(initialHoldings: string, allocations: ExitFormat.Allocation[], // we must index this with a JS number that is less than 2**32 - 1
indices: number[]): {
    newAllocations: ExitFormat.Allocation[];
    allocatesOnlyZeros: boolean;
    exitAllocations: ExitFormat.Allocation[];
    totalPayouts: string;
};
