"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createERC20DepositTransaction = exports.createETHDepositTransaction = exports.NitroAdjudicatorContractInterface = void 0;
const ethers_1 = require("ethers");
const NitroAdjudicator_json_1 = __importDefault(require("../../../artifacts/contracts/NitroAdjudicator.sol/NitroAdjudicator.json"));
exports.NitroAdjudicatorContractInterface = new ethers_1.ethers.utils.Interface(NitroAdjudicator_json_1.default.abi);
/**
 * Crafts an ethers TransactionRequest targeting the adjudicator's deposit method, specifically for ETH deposits
 * @param destination The channelId to deposit into
 * @param expectedHeld The amount you expect to have already been deposited
 * @param amount The amount you intend to deposit
 * @returns the transaction request
 */
function createETHDepositTransaction(destination, expectedHeld, amount) {
    const data = exports.NitroAdjudicatorContractInterface.encodeFunctionData('deposit', [
        ethers_1.constants.AddressZero,
        destination,
        expectedHeld,
        amount,
    ]);
    return { data };
}
exports.createETHDepositTransaction = createETHDepositTransaction;
/**
 * Crafts an ethers TransactionRequest targeting the adjudicator's deposit method, specifically for ERC20 deposits
 * @param tokenAddress The ERC20 token contract address
 * @param destination The channelId to deposit into
 * @param expectedHeld The amount you expect to have already been deposited
 * @param amount The amount you intend to deposit
 * @returns the transaction request
 */
function createERC20DepositTransaction(tokenAddress, destination, expectedHeld, amount) {
    const data = exports.NitroAdjudicatorContractInterface.encodeFunctionData('deposit', [
        tokenAddress,
        destination,
        expectedHeld,
        amount,
    ]);
    return { data };
}
exports.createERC20DepositTransaction = createERC20DepositTransaction;
