"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createTransferAllAssetsTransaction = exports.createConcludeAndTransferAllAssetsTransaction = exports.concludeAndTransferAllAssetsArgs = void 0;
const ethers_1 = require("ethers");
const NitroAdjudicator_json_1 = __importDefault(require("../../../artifacts/contracts/NitroAdjudicator.sol/NitroAdjudicator.json"));
const __1 = require("../../");
const outcome_1 = require("../outcome");
const state_1 = require("../state");
// https://github.com/ethers-io/ethers.js/issues/602#issuecomment-574671078
const NitroAdjudicatorContractInterface = new ethers_1.utils.Interface(NitroAdjudicator_json_1.default.abi);
function concludeAndTransferAllAssetsArgs(states, signatures, whoSignedWhat
// eslint-disable-next-line @typescript-eslint/no-explicit-any
) {
    // Sanity checks on expected lengths
    if (states.length === 0) {
        throw new Error('No states provided');
    }
    const { participants } = states[0];
    if (participants.length !== signatures.length) {
        throw new Error(`Participants (length:${participants.length}) and signatures (length:${signatures.length}) need to be the same length`);
    }
    const fixedPart = (0, state_1.getFixedPart)(states[0]);
    const variableParts = states.map(s => (0, state_1.getVariablePart)(s));
    const { proof, candidate } = (0, state_1.separateProofAndCandidate)((0, __1.bindSignatures)(variableParts, signatures, whoSignedWhat));
    return [fixedPart, proof, candidate];
}
exports.concludeAndTransferAllAssetsArgs = concludeAndTransferAllAssetsArgs;
function createConcludeAndTransferAllAssetsTransaction(states, signatures, whoSignedWhat) {
    return {
        data: NitroAdjudicatorContractInterface.encodeFunctionData('concludeAndTransferAllAssets', concludeAndTransferAllAssetsArgs(states, signatures, whoSignedWhat)),
    };
}
exports.createConcludeAndTransferAllAssetsTransaction = createConcludeAndTransferAllAssetsTransaction;
function transferAllAssetsArgs(state, overrideStateHash = false // set to true if channel concluded happily
// eslint-disable-next-line @typescript-eslint/no-explicit-any
) {
    const channelId = (0, __1.getChannelId)((0, state_1.getFixedPart)(state));
    const outcomeBytes = (0, outcome_1.encodeOutcome)(state.outcome);
    const stateHash = overrideStateHash ? ethers_1.constants.HashZero : (0, __1.hashState)(state);
    return [channelId, outcomeBytes, stateHash];
}
function createTransferAllAssetsTransaction(state, overrideStateHash = false // set to true if channel concluded happily
) {
    return {
        data: NitroAdjudicatorContractInterface.encodeFunctionData('transferAllAssets', transferAllAssetsArgs(state, overrideStateHash)),
    };
}
exports.createTransferAllAssetsTransaction = createTransferAllAssetsTransaction;
