import { BigNumberish } from 'ethers';
import { Outcome } from './contract/outcome';
/**
 * A mapping from destination to BigNumberish. E.g. {ALICE:2, BOB:3}. Only used in testing.
 */
export interface AssetOutcomeShortHand {
    [destination: string]: BigNumberish;
}
/**
 * A mapping from asset to AssetOutcomeShorthand. E.g. {ETH: {ALICE:2, BOB:3}, DAI: {ALICE:1, BOB:4}}. Only used in testing.
 */
export interface OutcomeShortHand {
    [assetHolder: string]: AssetOutcomeShortHand;
}
export interface AddressesLookup {
    [shorthand: string]: string | undefined;
}
/**
 * Recursively replaces any key in a copy of the supplied object with the value of that key in the supplied addresses object. Also BigNumberifies all numbers.
 * Used in testing only.
 * @param object Object to be copied and modified
 * @param addresses Key-value address lookup
 * @returns suitably modified copy of object
 */
export declare function replaceAddressesAndBigNumberify(object: AssetOutcomeShortHand | OutcomeShortHand | BigNumberish, addresses: AddressesLookup): AssetOutcomeShortHand | OutcomeShortHand | BigNumberish;
/** Computes an Outcome from a shorthand description */
export declare function computeOutcome(outcomeShortHand: OutcomeShortHand): Outcome;
export declare function getRandomNonce(seed: string): string;
