"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getRandomNonce = exports.computeOutcome = exports.replaceAddressesAndBigNumberify = void 0;
const exit_format_1 = require("@statechannels/exit-format");
const ethers_1 = require("ethers");
const bignumber_1 = require("@ethersproject/bignumber/lib/bignumber");
/**
 * Recursively replaces any key in a copy of the supplied object with the value of that key in the supplied addresses object. Also BigNumberifies all numbers.
 * Used in testing only.
 * @param object Object to be copied and modified
 * @param addresses Key-value address lookup
 * @returns suitably modified copy of object
 */
function replaceAddressesAndBigNumberify(object, addresses) {
    if ((0, bignumber_1.isBigNumberish)(object)) {
        return ethers_1.BigNumber.from(object);
    }
    const newObject = {};
    Object.keys(object).forEach(key => {
        if ((0, bignumber_1.isBigNumberish)(object[key])) {
            newObject[addresses[key]] = ethers_1.BigNumber.from(object[key]);
        }
        else if (typeof object[key] === 'object') {
            // Recurse
            newObject[addresses[key]] = replaceAddressesAndBigNumberify(object[key], addresses);
        }
    });
    return newObject;
}
exports.replaceAddressesAndBigNumberify = replaceAddressesAndBigNumberify;
/** Computes an Outcome from a shorthand description */
function computeOutcome(outcomeShortHand) {
    const outcome = [];
    Object.keys(outcomeShortHand).forEach(asset => {
        const allocations = [];
        Object.keys(outcomeShortHand[asset]).forEach(destination => allocations.push({
            destination,
            amount: ethers_1.BigNumber.from(outcomeShortHand[asset][destination]).toHexString(),
            metadata: '0x',
            allocationType: exit_format_1.AllocationType.simple,
        }));
        outcome.push({ asset, assetMetadata: { assetType: 0, metadata: '0x' }, allocations });
    });
    return outcome;
}
exports.computeOutcome = computeOutcome;
function getRandomNonce(seed) {
    // Returns a hex string representing a 64 bit integer
    return ethers_1.ethers.utils.id(seed).slice(0, 18); // '0x' plus [16 hexits is 8 bytes is 64 bits]
}
exports.getRandomNonce = getRandomNonce;
