"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createSignatureArguments = exports.createTransferAllAssetsTransaction = exports.createConcludeAndTransferAllAssetsTransaction = exports.createERC20DepositTransaction = exports.createETHDepositTransaction = exports.createConcludeTransaction = exports.createCheckpointTransaction = exports.createChallengeTransaction = exports.NITRO_MAX_GAS = exports.MAX_TX_DATA_SIZE = exports.MAGIC_ADDRESS_INDICATING_ETH = void 0;
const ethers_1 = require("ethers");
const forceMoveTrans = __importStar(require("./contract/transaction-creators/force-move"));
const multiAssetHolderTrans = __importStar(require("./contract/transaction-creators/multi-asset-holder"));
const nitroAdjudicatorTrans = __importStar(require("./contract/transaction-creators/nitro-adjudicator"));
const signatures_1 = require("./signatures");
// CONSTANTS
exports.MAGIC_ADDRESS_INDICATING_ETH = ethers_1.constants.AddressZero;
exports.MAX_TX_DATA_SIZE = 128 * 1024; // (bytes) https://github.com/ethereum/go-ethereum/blob/f59ed3565d18c1fa676fd34f4fd41ecccad707e8/core/tx_pool.go#L51
exports.NITRO_MAX_GAS = 6000000; // should be below the block gas limit, which can change over time and is generally different on mainnet, testnet and ganache.
// sampling some recent blocks on 26/11/2020:
// mainnet  12505858
// ropsten:  8000029
// rinkeby: 10000000
// ganache:  6721975 (hardcoded but can be overriden via config)
/**
 * Marshalls the supplied signedStates into an ethereum transaction for the challenge method. Automatically computes signedVariableParts, etc.
 * @param signedStates an array of signed states
 * @param challengePrivateKey private key of the challenger
 * @returns An ethers TransactionRequest. This can be launched with `await signer.sendTransaction({to: adjudicator.address, ...txRequest}`)
 */
function createChallengeTransaction(signedStates, challengePrivateKey) {
    const { states, signatures, whoSignedWhat } = createSignatureArguments(signedStates);
    return forceMoveTrans.createChallengeTransaction(states, signatures, whoSignedWhat, challengePrivateKey);
}
exports.createChallengeTransaction = createChallengeTransaction;
/**
 * Marshalls the supplied signedStates into an ethereum transaction for the checkpoint method. Automatically computes signedVariableParts, etc.
 * @param signedStates an array of signed states
 * @returns An ethers TransactionRequest. This can be launched with `await signer.sendTransaction({to: adjudicator.address, ...txRequest}`)
 */
function createCheckpointTransaction(signedStates) {
    const { states, signatures, whoSignedWhat } = createSignatureArguments(signedStates);
    return forceMoveTrans.createCheckpointTransaction({
        states,
        signatures,
        whoSignedWhat,
    });
}
exports.createCheckpointTransaction = createCheckpointTransaction;
/**
 * Marshalls the supplied signedStates into an ethereum transaction for the conclude method. Automatically computes signedVariableParts, etc.
 * @param conclusionProof an array of signed states
 * @returns An ethers TransactionRequest. This can be launched with `await signer.sendTransaction({to: adjudicator.address, ...txRequest}`)
 */
function createConcludeTransaction(conclusionProof) {
    const { states, signatures, whoSignedWhat } = createSignatureArguments(conclusionProof);
    return forceMoveTrans.createConcludeTransaction(states, signatures, whoSignedWhat);
}
exports.createConcludeTransaction = createConcludeTransaction;
/**
 * Marshalls the supplied destination, expectedHeld and amount into an ethereum transaction for the deposit ETH method.
 * @param destination The channelId to deposit into
 * @param expectedHeld The amount you expect to have already been deposited
 * @param amount The amount you intend to deposit
 * @returns the transaction request
 */
function createETHDepositTransaction(destination, expectedHeld, amount) {
    return multiAssetHolderTrans.createETHDepositTransaction(destination, expectedHeld, amount);
}
exports.createETHDepositTransaction = createETHDepositTransaction;
/**
 * Marshalls the supplied tokenAddress, destination, expectedHeld and amount into an ethereum transaction for the deposit ERC20 token method.
 * @param tokenAddress The ERC20 token contract address
 * @param destination The channelId to deposit into
 * @param expectedHeld The amount you expect to have already been deposited
 * @param amount The amount you intend to deposit
 * @returns the transaction request
 */
function createERC20DepositTransaction(tokenAddress, destination, expectedHeld, amount) {
    return multiAssetHolderTrans.createERC20DepositTransaction(tokenAddress, destination, expectedHeld, amount);
}
exports.createERC20DepositTransaction = createERC20DepositTransaction;
/**
 * Marshalls the supplied signedStates into an ethereum transaction for the conclude and transfer all assets method. Automatically computes signedVariableParts, etc.
 * @param signedStates an array of signed states
 * @returns An ethers TransactionRequest. This can be launched with `await signer.sendTransaction({to: adjudicator.address, ...txRequest}`)
 */
function createConcludeAndTransferAllAssetsTransaction(signedStates) {
    const { states, signatures, whoSignedWhat } = createSignatureArguments(signedStates);
    return nitroAdjudicatorTrans.createConcludeAndTransferAllAssetsTransaction(states, signatures, whoSignedWhat);
}
exports.createConcludeAndTransferAllAssetsTransaction = createConcludeAndTransferAllAssetsTransaction;
/**
 * Marshalls the supplied signedStates into an ethereum transaction for the transfer all assets method. Automatically computes signedVariableParts, etc.
 * @param state last state in the channel
 * @returns An ethers TransactionRequest. This can be launched with `await signer.sendTransaction({to: adjudicator.address, ...txRequest}`)
 */
function createTransferAllAssetsTransaction(state) {
    return nitroAdjudicatorTrans.createTransferAllAssetsTransaction(state);
}
exports.createTransferAllAssetsTransaction = createTransferAllAssetsTransaction;
/**
 * Marshalls the supplied signedStates into the signature arguments used in most on chain methods.
 * Currently we assume each signedState is a unique combination of state/signature
 * So if multiple participants sign a state we expect a SignedState for each participant
 * @param signedStates an array of signed states
 * @returns Object with (states, signatures, whosignedWhat)
 */
function createSignatureArguments(signedStates) {
    const { participants } = signedStates[0].state;
    const states = [];
    const whoSignedWhat = new Array(participants.length);
    // Get a list of all unique signed states.
    const uniqueSignedStates = signedStates.filter((s, i, a) => a.indexOf(s) === i);
    // We may receive multiple Signed States which have the same state and different signatures
    // so we get a list of unique states ignoring their signatures
    // which allows us to create a single state with multiple signatures
    const uniqueStates = uniqueSignedStates.map(s => s.state).filter((s, i, a) => a.indexOf(s) === i);
    const signatures = new Array(uniqueStates.length);
    for (let i = 0; i < uniqueStates.length; i++) {
        states.push(uniqueStates[i]);
        // Get a list of all signed states that have the state
        const signedStatesForUniqueState = uniqueSignedStates.filter(s => s.state === uniqueStates[i]);
        // Iterate through the signatures and set signatures/whoSignedWhawt
        for (const ss of signedStatesForUniqueState) {
            const participantIndex = participants.indexOf((0, signatures_1.getStateSignerAddress)(ss));
            signatures[participantIndex] = ss.signature;
            whoSignedWhat[participantIndex] = i;
        }
    }
    return {
        states,
        signatures,
        whoSignedWhat,
    };
}
exports.createSignatureArguments = createSignatureArguments;
